// This is a roxen module. (c) Martin Baehr 1999

// The group container module. 
// will group entities seperated by <sep> according to the group-size
// given

//  This code is (c) 1999 Martin Baehr, and can be used, modified and
//  redistributed freely under the terms of the GNU General Public License,
//  version 2.
//  This code comes on a AS-IS basis, with NO WARRANTY OF ANY KIND, either
//  implicit or explicit. Use at your own risk.
//  You can modify this code as you wish, but in this case please
//  - state that you changed the code in the modified version
//  - do not remove my name from it
//  - send me a copy of the modified version or a patch, so that
//    I can include it in the 'official' release.
//  If you find this code useful, please e-mail me. It would definitely
//  boost my ego :)
//  
//  For risks and side-effects please read the code or ask your local 
//  unix or roxen-guru.

constant cvs_version="$Id: group.pike,v 1.4 1999/09/25 02:22:24 mbaehr Exp $";

#include <module.h>
inherit "module";
inherit "roxenlib";

static private string doc()
{
  return "Defines the <tt>&lt;group&gt;&lt;/group&gt;</tt> container, "
         "which allows you to group a list of entities into chunks/groups.<p>\n"
         "the size of each chunk is defined with the argument "
         "<b>size</b>=<tt>number</tt><br>\n"
         "the entities are seperated with the tag <tt>&lt;sep&gt;</tt><br> "
         "each seperator will then succesively be replaced with the contents "
         "of the <tt>&lt;seperator&gt;&lt;/seperator&gt;</tt> subcontainer, "
         "every <tt>number</tt> <tt>&lt;sep&gt;</tt> is replaced with the "
         "contents of the "
         "<tt>&lt;group_seperator&gt;&lt;/group_seperator&gt;</tt> "
         " subcontainer.<p>\n"
         "Help can be displayed with <tt>&lt;group help&gt;&lt;/group&gt;</tt>";
}

array register_module()
{
  return ({ MODULE_PARSER, "Group",
            doc()
            ,0,1 });
}

string container_group(string name, 
                     mapping arguments, 
                     string contents, 
                     object id, 
                     mapping defines) 
{
  werror(sprintf("Group: %O\n", arguments));

  if(arguments->help)
    return("<obox><title>&lt;"+name+"&gt;&lt;/"+name+"&gt;</title>"+doc()+"</obox>");

  id->misc->_group_size=(int)arguments->size;
  id->misc->_group_count=1;

  contents = parse_html(contents, ([]), ([ "group_seperator":icontainer_args, "seperator":icontainer_args ]), id);

  return parse_html(parse_rxml(contents, id), ([ "sep":itag_sep ]), ([]), id);
}

string itag_sep(string tag, mapping arguments, object id)
{
  werror(sprintf("Group: %s:%d:%d\n", tag, id->misc->_group_count, id->misc->_group_size));
  if(id->misc->_group_count == id->misc->_group_size)
  {
    id->misc->_group_count = 1;
    return id->misc->_group_group_seperator||"";
  }
  else
  {
    id->misc->_group_count++;
    return id->misc->_group_seperator||"";
  }
}

string icontainer_args(string tag_name,mapping arguments,
                               string contents, object id)
{
  
  id->misc[ "_group_"+tag_name ] = contents;
  return "";
}

mapping query_container_callers()
{
  return ([ "group":container_group, ]);
}
